## Copyright (C) 2014 Fotios Kasolis
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or (at
## your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*- 
## @deftypefn {Function File} @var{B}= bandwidth (@var{A}, @var{luband})
## @deftypefnx {Function File} [@var{l,} @var{u} ] = bandwidth (@var{A})
##
## Return the lower and upper bandwidths of a numerical matrix @var{A}
##
## When used, the option @var{luband} attains the string values 'lower'
## and 'upper' for the lower and upper bandwidths, respectively. If
## called with two output arguments, the function returns both bandwidths.
##
## @seealso{diag, isdiag, isbanded}
## @end deftypefn

## Author: Fotios Kasolis <fotios.kasolis@gmail.com>
## Created: 2014-09-23

function [ varargout ] = bandwidth (varargin)

  if ( nargin < 1 || nargin > 2 )
    print_usage ();
  endif

  A = varargin{1};
  if ( !isnumeric (A) || !ismatrix (A) )
    error ("expexted numerical matrix input");
  endif
  
  [nr, nc] = size(A);
  if ( nargin == 2 )
    luband = varargin{2};
    if ( !ischar (luband) )
      error ("expexted string as second argument");
    endif
    if ( !strcmp (luband, "lower") && !strcmp (luband, "upper"))
      error ("available options are 'lower' and 'upper'")
    endif
 
    idx = 0;
    if ( strcmp (luband, "lower") )
      for cnt = 1 : nr
         if ( !isempty (find (tril (A, -cnt))) )
           idx = idx + 1;
         endif
      endfor
      varargout{1} = idx; 
    endif
    if ( strcmp (luband, "upper") )
      for cnt = 1 : nr
         if ( !isempty (find (triu (A, cnt))) )
           idx = idx + 1;
         endif
      endfor
      varargout{1} = idx; 
    endif
  endif
  if ( nargin == 1 )
    idxl = 0;
    idxu = 0;
    for cnt = 1 : nr
      if ( !isempty (find (tril (A, -cnt))) )
        idxl = idxl + 1;
      endif
    endfor
    for cnt = 1 : nr
      if ( !isempty (find (triu (A, cnt))) )
        idxu = idxu + 1;
      endif
    endfor
    varargout{1} = idxl;
    varargout{2} = idxu;
  endif
    
endfunction

%!test
%! assert (isequal (bandwidth (eye(9), "lower"), 0));
%! assert (isequal (bandwidth (eye(9), "upper"), 0));
%! assert (isequal (bandwidth (toeplitz ([1,2,3,0,0,0]), "lower"), 2));
%! assert (isequal (bandwidth (toeplitz ([1,2,3,0,0,0]), "upper"), 2));
%! assert (isequal (bandwidth (triu (rand (9)), "lower"), 0));
%! assert (isequal (bandwidth (triu (rand (9)), "upper"), 8));
%! [l, u] = bandwidth (ones (9));
%! assert (isequal (l, 8));
%! assert (isequal (u, l));